#ifndef _TULIP_DRIVER_H
#define _TULIP_DRIVER_H

#include <tulip.h>

struct recv_descriptor {
	union {
		struct {
#ifdef BIG_ENDIAN
			uint32_t own:1;
			uint32_t fl:15;
			uint32_t es:1;
			uint32_t le:1;
			uint32_t dt:2;
			uint32_t rf:1;
			uint32_t mf:1;
			uint32_t fs:1;
			uint32_t ls:1;
			uint32_t tl:1;
			uint32_t cs:1;
			uint32_t ft:1;
			uint32_t rj:1;
			uint32_t pad:1;
			uint32_t db:1;
			uint32_t ce:1;
			uint32_t of:1;
#else
			uint32_t of:1;
			uint32_t ce:1;
			uint32_t db:1;
			uint32_t pad:1;
			uint32_t rj:1;
			uint32_t ft:1;
			uint32_t cs:1;
			uint32_t tl:1;
			uint32_t ls:1;
			uint32_t fs:1;
			uint32_t mf:1;
			uint32_t rf:1;
			uint32_t dt:2;
			uint32_t le:1;
			uint32_t es:1;
			uint32_t fl:15;
			uint32_t own:1;
#endif
		} x;
		uint32_t raw;
	} rdes0;

	union {
		struct {
#ifdef BIG_ENDIAN
			uint32_t pad1:6;
			uint32_t rer:1;
			uint32_t rch:1;
			uint32_t pad2:2;
			uint32_t rbs2:11;
			uint32_t rbs1:11;
#else
			uint32_t rbs1:11;
			uint32_t rbs2:11;
			uint32_t pad2:2;
			uint32_t rch:1;
			uint32_t rer:1;
			uint32_t pad1:6;
#endif
		} x;
		uint32_t raw;
	} rdes1;

	uint32_t rdes2;
	uint32_t rdes3;
};

struct trans_descriptor {
	union {
		struct {
#ifdef BIG_ENDIAN
			uint32_t own:1;
			uint32_t pad1:15;
			uint32_t es:1;
			uint32_t to:1;
			uint32_t pad2:2;
			uint32_t lo:1;
			uint32_t nc:1;
			uint32_t lc:1;
			uint32_t ec:1;
			uint32_t hf:1;
			uint32_t cc:4;
			uint32_t lf:1;
			uint32_t uf:1;
			uint32_t de:1;
#else
			uint32_t de:1;
			uint32_t uf:1;
			uint32_t lf:1;
			uint32_t cc:4;
			uint32_t hf:1;
			uint32_t ec:1;
			uint32_t lc:1;
			uint32_t nc:1;
			uint32_t lo:1;
			uint32_t pad2:2;
			uint32_t to:1;
			uint32_t es:1;
			uint32_t pad1:15;
			uint32_t own:1;
#endif
		} x;
		uint32_t raw;
	} tdes0;

	union {
		struct {
#ifdef BIG_ENDIAN
			uint32_t ic:1;
			uint32_t ls:1;
			uint32_t fs:1;
			uint32_t ft1:1;
			uint32_t set:1;
			uint32_t ac:1;
			uint32_t ter:1;
			uint32_t tch:1;
			uint32_t dpd:1;
			uint32_t ft0:1;
			uint32_t tbs2:11;
			uint32_t tbs1:11;
#else
			uint32_t tbs1:11;
			uint32_t tbs2:11;
			uint32_t ft0:1;
			uint32_t dpd:1;
			uint32_t tch:1;
			uint32_t ter:1;
			uint32_t ac:1;
			uint32_t set:1;
			uint32_t ft1:1;
			uint32_t fs:1;
			uint32_t ls:1;
			uint32_t ic:1;
#endif
		} x;
		uint32_t raw;
	} tdes1;

	uint32_t tdes2;
	uint32_t tdes3;
};

#define TX_NUMDESC	10
#define RX_NUMDESC	8
#define RX_DESCLEN	1536

enum rcv_state {STOPPED, RUNNING, SUSPENDED};

/* Device driver data */
struct tulip {
	void			*cthis;
	bus_space_t		csr, pciconf;
	dma_handle_t		dma, tx_dma;

	uint8_t			rombuf[128];
	uint64_t		mac_addr;

	/* receive buffers and descriptors */
	mem_handle_t		rcv_buffer_handle[RX_NUMDESC];
	void			*rcv_buffer_pointer[RX_NUMDESC];
	dma_addr_t		rcv_buffer_dma[RX_NUMDESC];

	mem_handle_t		rcv_ring_handle;
	struct recv_descriptor	*rcv_ring;
	dma_addr_t		rcv_ring_dma;
	int			rcv_ring_end_pos;
	int			rcv_ring_start_pos;

	/*lock_t			rcv_lock;*/
	enum rcv_state		rcv_state;
	uint32_t		rcv_keys[RX_NUMDESC];
	int			rcv_full;

	/* transmit buffers and descriptors */
	mem_handle_t		send_buffer_handle[TX_NUMDESC];
	void			*send_buffer_pointer[TX_NUMDESC];
	dma_addr_t		send_buffer_dma[TX_NUMDESC];
	uint64_t		transmit_keys[TX_NUMDESC];

	mem_handle_t		send_ring_handle;
	struct trans_descriptor	*send_ring;
	dma_addr_t		send_ring_dma;
	int			send_ring_end_pos;
	int			send_ring_start_pos;
	int			send_frame_start_pos;

	void *                  callback_data;
	void (*tx_freepacket)(void *, uint32_t, uint64_t);
	int (*rx_feedpacket)(void *, uint32_t, size_t, uint64_t);
};

#define SROM_BITWIDTH	6
#define SROMCMD_RD	6

#endif
